@@ -315,61 +315,60 @@ list_adls_files <- function(filesystem, dir="/", info=c("all", "name"),
     {
         res <- do_container_op(filesystem, "", options=opts, http_status_handler="pass")
         httr::stop_for_status(res, storage_error_message(res))
-        dfchunk <- httr::content(res, simplifyVector=TRUE)$paths
-
-        # normalise output
-        if(is.null(dfchunk$isDirectory))
-            dfchunk$isDirectory <- FALSE
-        else dfchunk$isDirectory <- !is.na(dfchunk$isDirectory)
-        if(is.null(dfchunk$contentLength))
-            dfchunk$contentLength <- NA_real_
-        else dfchunk$contentLength[is.na(dfchunk$contentLength)] <- NA_real_
-        if(is.null(dfchunk$etag))
-            dfchunk$etag <- ""
-        else dfchunk$etag[is.na(dfchunk$etag)] <- ""
-        if(is.null(dfchunk$permissions))
-            dfchunk$permissions <- ""
-        else dfchunk$permissions[is.na(dfchunk$permissions)] <- ""
-        if(is.null(dfchunk$lastModified))
-            dfchunk$lastModified <- ""
-
-        dfchunk <- dfchunk[c("name", "contentLength", "isDirectory", "lastModified", "permissions", "etag")]
-        if(!is.null(dfchunk$contentLength))
-            dfchunk$contentLength <- as.numeric(dfchunk$contentLength)
-        if(!is.null(dfchunk$lastModified))
-            dfchunk$lastModified <- as_datetime(dfchunk$lastModified)
-        names(dfchunk)[c(2, 3)] <- c("size", "isdir")
-
-        if(all(dfchunk$permissions == ""))
-            dfchunk$permissions <- NULL
-        if(all(dfchunk$etag == ""))
-            dfchunk$etag <- NULL
-
-        # needed when dir was created in a non-HNS enabled account
-        dfchunk$size[dfchunk$isdir] <- NA
-
-        out <- rbind(out, dfchunk)
+        out <- vctrs::vec_rbind(out, httr::content(res, simplifyVector=TRUE)$paths)
         headers <- httr::headers(res)
         if(is_empty(headers$`x-ms-continuation`))
             break
         else opts$continuation <- headers$`x-ms-continuation`
     }
 
-    if(info == "all")
+    # cater for null output
+    if(is_empty(out))
     {
-        # cater for null output
-        if(is_empty(out))
-            return(data.frame(
-                name=character(0),
-                size=numeric(0),
-                isdir=logical(0),
-                lastModified=structure(numeric(0), class=c("POSIXct", "POSIXt"), tzone="GMT"),
-                permissions=character(0),
-                etag=character(0)))
-
-        out
+        out <- if(info == "name")
+            character(0)
+        else data.frame(
+            name=character(0),
+            size=numeric(0),
+            isdir=logical(0),
+            lastModified=structure(numeric(0), class=c("POSIXct", "POSIXt"), tzone="GMT"),
+            permissions=character(0),
+            etag=character(0))
+        return(out)
     }
-    else as.character(out$name)
+    else if(info == "name")
+        return(out$name)
+
+    # normalise output
+    if(is.null(out$isDirectory))
+        out$isDirectory <- FALSE
+    else out$isDirectory <- !is.na(out$isDirectory)
+    if(is.null(out$contentLength))
+        out$contentLength <- NA_real_
+    if(is.null(out$etag))
+        out$etag <- ""
+    else out$etag[is.na(out$etag)] <- ""
+    if(is.null(out$permissions))
+        out$permissions <- ""
+    else out$permissions[is.na(out$permissions)] <- ""
+    if(is.null(out$lastModified))
+        out$lastModified <- ""
+
+    out <- out[c("name", "contentLength", "isDirectory", "lastModified", "permissions", "etag")]
+    if(!is.null(out$contentLength))
+        out$contentLength <- as.numeric(out$contentLength)
+    if(!is.null(out$lastModified))
+        out$lastModified <- as_datetime(out$lastModified)
+    names(out)[c(2, 3)] <- c("size", "isdir")
+
+    if(all(out$permissions == ""))
+        out$permissions <- NULL
+    if(all(out$etag == ""))
+        out$etag <- NULL
+
+    # needed when dir was created in a non-HNS enabled account
+    out$size[out$isdir] <- NA
+    out
 }
 
 
@@ -451,6 +450,19 @@ delete_adls_dir <- function(filesystem, dir, recursive=FALSE, confirm=TRUE)
     if(!delete_confirmed(confirm, paste0(filesystem$endpoint$url, filesystem$name, "/", dir), "directory"))
         return(invisible(NULL))
 
+    # special-case handling of deletion of root dir
+    if(dir == "/" && recursive)
+    {
+        lst <- list_adls_files(filesystem, "/", info="all")
+        for(i in seq_len(nrow(lst)))
+        {
+            if(lst$isdir[i])
+                delete_adls_dir(filesystem, lst$name[i], recursive=TRUE, confirm=FALSE)
+            else delete_adls_file(filesystem, lst$name[i], confirm=FALSE)
+        }
+        return(invisible(NULL))
+    }
+
     opts <- list(recursive=tolower(as.character(recursive)))
     invisible(do_container_op(filesystem, dir, options=opts, http_verb="DELETE"))
 }
